#ifndef DAS20_HH
#define	DAS20_HH

#ifndef __GNUC__
#define __inline__ inline
#define __const__ const
#endif

#define BASE_ADDR       0x300             /* DAS-20 base address */
#define ADC_LO          BASE_ADDR + 0x00  /* ADC low byte (R) */
#define ADC_START       BASE_ADDR + 0x00  /* ADC manual start (W) */
#define ADC_HI          BASE_ADDR + 0x01  /* ADC high byte (R) */
#define SCANQ_VIEW      BASE_ADDR + 0x02  /* Scanner queue view (R) */        
#define SCANQ_LOAD      BASE_ADDR + 0x02  /* Scanner queue load (W) */    
#define CONTROL         BASE_ADDR + 0x03  /* DAS-20 control (RW) */
#define INT_STATUS      BASE_ADDR + 0x04  /* Interrupt status (R) */         
#define INT_CONTROL     BASE_ADDR + 0x04  /* Interrupt control (W) */        
#define DMA_CONTROL     BASE_ADDR + 0x05  /* DMA control (RW) */             
#define TMR_DATA        BASE_ADDR + 0x06  /* Counter/Timer data (RW) */
#define TMR_CONTROL     BASE_ADDR + 0x07  /* Counter/Timer control (RW) */
#define DAC0_LO         BASE_ADDR + 0x08  /* DAC #0 low byte (W) */          
#define DAC0_HI         BASE_ADDR + 0x09  /* DAC #0 high byte (W) */
#define DAC1_LO         BASE_ADDR + 0x0A  /* DAC #1 low byte (W) */
#define DIGITAL_IN      BASE_ADDR + 0x0B  /* Digital input (R) */           
#define DAC1_HI         BASE_ADDR + 0x0B  /* DAC #1 high byte (W) */
#define DIGITAL_OUT     BASE_ADDR + 0x0C  /* Digital output (RW) */

/* Ranges for AD conversions */
#define RANGE1       0x00     /* 0 to +10 Volts */
#define RANGE2       0x02     /* -10 to +10 Volts */
#define RANGE3       0x04     /* 0 to +10 Volts */
#define RANGE4       0x06     /* -5 to +5 Volts */
#define RANGE5       0x08     /* 0 to +1 Volt */
#define RANGE6       0x0A     /* -0.5 to +0.5 Volts */
#define RANGE7       0x0C     /* 0 to +100mV */
#define RANGE8       0x0E     /* -50mV to +50mV */
#define QUEUE_END    0x01     /* End of ADC queue */

/* ADC register commands */
#define ADC_Q_ENABLE    0x80     /* Enable ADC queue */
#define ADC_Q_DISABLE   0x00
#define ADC_Q_WRAP      0x40     /* ADC queue wraps around at end */
#define ADC_Q_LOAD      0x40     /* Load ADC queue */
#define ADC_Q_STOP      0x00     /* ADC queue stops at end */
#define DMA_ACTIVE      0x20     /* Activate DMA requests */
#define DMA_NOT_ACTIVE  0x00
#define TIMER_ENABLE    0x10     /* Enable timer to delay ADC start */
#define TIMER_DISABLE   0x00
#define TRIG_HARDWARE   0x08     /* Trigger for ADC is hardware */
#define TRIG_SOFTWARE   0x00
#define TRIG_EXT        0x04     /* Hardware trigger is external */
#define TRIG_TMR5        0x00     /* Hardware trigger is timer 5 TC */
#define ADC_GATE_EXT    0x02     /* ADC is gated externally */
#define ADC_GATE_INT    0x00     /* ADC is gated by EOC status */
#define ADC_GATE_HI     0x01     /* Gate is active high */
#define ADC_GATE_LO      0x00

/* Interrupt register commands */
#define INT_ENABLE      0x80     /* An interrupt event causes a system IRQ */
#define INT_DISABLE     0x00
#define INT_LEVEL2      0x20     /* Set IRQ level */
#define INT_LEVEL3      0x30
#define INT_LEVEL4      0x40
#define INT_LEVEL5      0x50
#define INT_LEVEL6      0x60
#define INT_LEVEL7      0x70
#define INT_ADC_EOC     0x00     /* IRQ source is ADC End of Conversion */
#define INT_TMR2        0x04     /* IRQ source is timer 2 TC */
#define INT_Q_END       0x08     /* IRQ source is end of ADC queue */
#define INT_DMA_TC      0x06     /* IRQ source is DMA TC */
#define INT_MASK        0x02     /* Interrupt pending mask */

/* DMA register commands */

#define DMA_ENABLE      0x80
#define DMA_DISABLE     0x00
#define DMA_LEVEL3      0x40
#define DMA_LEVEL1      0x00
#define DMA_ADC_EOC     0x00 /* DMA xfer initiated by end of conversion */
#define DMA_DAC0_TMR2   0x10 /* 2 byte DAC0 xfer initiated by tmr 2 TC */
#define DMA_DAC1_TMR2   0x20 /* 2 byte DAC1 xfer initiated by tmr 2 TC */
#define DMA_DAC01_TMR2  0x30 /* 4 byte DAC0,1 xfer initiated by tmr 2 TC */


/*************************************************************************
 * Mneumonics for the AMD9513A system timer
 **************************************************************************/
/* Command register mnemonics
 **************************************************************************/
/* Commands that affect the data pointer */
/* Usage "_outp(TMR_CONTROL, (LOAD_DPR + group + element));" */
#define LOAD_DPR        0x00
/* Groups */
#define GROUP1          0x01
#define GROUP2          0x02
#define GROUP3          0x03
#define GROUP4          0x04
#define GROUP5          0x05
#define CONTROL_GROUP   0x07

/* Elements for group# */
#define MODE_REG        0x00
#define LOAD_REG        0x08
#define HOLD_REG        0x10
#define HOLD_REG_CYCLE  0x18  /* Hold registers in the hold reg cycle */
/* Elements for control group */
#define ALARM_REG1      0x00
#define ALARM_REG2      0x08
#define MASTER_MODE_REG 0x10
#define STATUS_REG      0x18  /* No cycle. May also read TMR_CONTROL */


/* Commands that effect multiple timers */
/* Usage "_outp(TMR_CONTROL, (command + timer# + timer# + ...));" */

#define ARM_TMR         0x20
#define LOAD_TMR        0x40
#define LOAD_ARM_TMR    0x60
#define DISARM_TMR      0xC0
#define SAVE_TMR        0xA0
#define DISARM_SAVE_TMR 0x80
/* Timer numbers */
#define TIMER1          0x01
#define TIMER2          0x02
#define TIMER3          0x04
#define TIMER4          0x08
#define TIMER5          0x10


/* Commands that effect single timers */
/* Usage "_outp(TMR_CONTROL, (command + group#);" */
/* Group numbers are defined above. CONTROL_GROUP is not valid here. */
#define STEP_TMR        0xF0
#define SET_TC_TOGGLE   0xE8
#define CLEAR_TC_TOGGLE 0xE0


/* Commands that need no other arguements */
/* Usage "_outp(TMR_CONTROL, (command));" */
#define DISABLE_DPR_SEQ    0xE8     /* Disable automatic data pointer sequence */
#define ENABLE_DPR_SEQ     0xE0
#define ENABLE_16_BIT      0xEF     /* DO NOT USE THIS!!! */
#define ENABLE_8_BIT       0xE7
#define GATE_OFF_FOUT      0xEE     /* FOUT output is turned off */
#define GATE_ON_FOUT       0xE6
#define DISABLE_PREFETCH   0xF9     /* Disable prefetch for write operations */
#define ENABLE_PREFETCH    0xF8
#define MASTER_RESET       0xFF     /* Equivalent to a power on reset */


/* Counter mode register mnemonics */
/**************************************************************************/

/* The counter mode registers are 16 bit registers that */
/* must be loaded in two 8 bit bytes in low-byte */
/* high-byte order. */
/* Usage: " _outp(TMR_CONTROL, (LOAD_DPR + group + MODE_REG); */
/*          _outp(TMR_DATA, (<low byte options>)); */
/*          _outp(TMR_DATA, (<high byte options>)); " */

/* High Byte */

/* Gating control */

#define NO_GATE         0x00  /* No gating */
#define ACT_HI_TC_NM1   0x20  /* Active hi gating from TC of COUNTER n-1 */
#define ACT_HI_GATE_NP1 0x40  /* Active hi gating from GATE n+1 */
#define ACT_HI_GATE_NM1 0x60  /* Active hi gating from GATE n-1 */
#define ACT_HI_GATE_N   0x80  /* Active hi gating from GATE n */
#define ACT_LO_GATE_N   0xA0  /* Active lo gating from GATE n */
#define ACT_RISE_GATE_N 0xC0  /* Active rising edge gating from GATE n */
#define ACT_FALL_GATE_N 0xE0  /* Active falling edge gating from GATE n */

/* Source edge */
#define RISE_SOURCE     0x00  /* Source counts on rising/falling edge */
#define FALL_SOURCE     0x10

/* Count source selection */
#define SOURCE_TC_NM1   0x00  /* Source is TC of COUNTER n-1 */
#define SOURCE1         0x01  /* Source is SRC1 */
#define SOURCE2         0x02  /* Source is SRC2 */
#define GATE1           0x06  /* Source is GATE1 */
#define GATE2           0x07  /* Source is GATE2 */
#define F1              0x0B  /* 5MHz clock */
#define F2              0x0C  /* 500KHz if MM15=1 else 312.50KHz */
#define F3              0x0D  /* 50KHz  if MM15=1 else  19.53KHz */
#define F4              0x0E  /* 5KHz   if MM15=1 else 1220.7Hz */
#define F5              0x0F  /* 500Hz  if MM15=1 else   76.3Hz */

/* Low Byte */

/* Count control */
#define DISABLE_SP_GATE 0x00  /* Enable/disable special gating functions */
#define ENABLE_SP_GATE  0x80

#define RELOAD_LOAD     0x00  /* Reload from load */
#define RELOAD_EITHER   0x40  /* Reload from load or hold except in mode X */

#define COUNT_ONCE      0x00  /* Count only once */
#define COUNT_REPEAT    0x20  /* Count repetitively */

#define BINARY_COUNT    0x00  /* Count in binary */
#define BCD_COUNT       0x10  /* Count in BCD */

#define COUNT_DOWN      0x00  /* Count up/down */
#define COUNT_UP        0x08

/* Output control */
#define INACTIVE_LO     0x00  /* Output is inactive and held low */
#define ACT_HI_TC_PULSE 0x01  /* Active high Terminal Count (TC) pulse */
#define TC_TOGGLE       0x02  /* TC toggled output */
#define INACTIVE_HI_Z   0x04  /* Output is inactive and in Hi-Z state */
#define ACT_LO_TC_PULSE 0x05  /* Active low TC pulse */


/* Master mode register mnemonics */
/**************************************/

/* High Byte */

/* Scalar control */
#define BINARY_DIVISION 0x00
#define BCD_DIVISION    0x80

/* Data pointer control */
#define ENABLE_INC      0x00
#define DISABLE_INC     0x40

/* Data bus width */
#define BUS_8_BIT       0x00
#define BUS_16_BIT      0x20

/* FOUT gate */
#define FOUT_ON         0x00
#define FOUT_OFF        0x10

/* FOUT divider */

#define DIVIDE_BY_16    0x00
#define DIVIDE_BY_15    0x0F
#define DIVIDE_BY_14    0x0E
#define DIVIDE_BY_13    0x0D
#define DIVIDE_BY_12    0x0C
#define DIVIDE_BY_11    0x0B
#define DIVIDE_BY_10    0x0A
#define DIVIDE_BY_9     0x09
#define DIVIDE_BY_8     0x08
#define DIVIDE_BY_7     0x07
#define DIVIDE_BY_6     0x06
#define DIVIDE_BY_5     0x05
#define DIVIDE_BY_4     0x04
#define DIVIDE_BY_3     0x03
#define DIVIDE_BY_2     0x02
#define DIVIDE_BY_1     0x01

/* Low Byte */

/* FOUT source */
#define FOUT_SRC_E1     0x00
#define FOUT_SRC_SRC1   0x01
#define FOUT_SRC_SRC2   0x02
#define FOUT_SRC_SRC3   0x03
#define FOUT_SRC_SRC4   0x04
#define FOUT_SRC_SRC5   0x05
#define FOUT_SRC_GATE1  0x06
#define FOUT_SRC_GATE2  0x07
#define FOUT_SRC_GATE3  0x08
#define FOUT_SRC_GATE4  0x09
#define FOUT_SRC_GATE5  0x0A
#define FOUT_SRC_F1     0x0B
#define FOUT_SRC_F2     0x0C
#define FOUT_SRC_F3     0x0D
#define FOUT_SRC_F4     0x0E
#define FOUT_SRC_F5     0x0F

/* Comparator 2 */
#define ENABLE_COMP2    0x08
#define DISABLE_COMP2   0x00

/* Comparator 1 */
#define ENABLE_COMP1    0x04
#define DISABLE_COMP1   0x00

/* Time of day mode */
#define DISABLE_TOD     0x00
#define TOD5            0x01
#define TOD6            0x02
#define TOD10           0x03



class DAS20CL
{
public:
  DAS20CL(void);
  ~DAS20CL(void);

private:
  int fdio_port;

public:
  /* Misc functions */
  void das20_master_reset(void);      
  
  /* Functions that access the digital ports */
  void set_digital_byte(char value);
  void set_digital_bit(short int bit, short int value);
  void toggle_digital_byte(void);
  void toggle_digital_bit(short int bit);
  char get_digital_out_byte(void);
  int get_digital_out_bit(short int bit);
  char get_digital_in_byte(void);
  int get_digital_in_bit(short int bit);
  
  /* Functions that deal with ADC */
  double single_adc(short int channel, short int range);
  void load_queue(char chan[], char range[], int number);
  
  /* Functions that dael with DAC */
  int single_dac(short int channel, double value);
  
  /* Functions that deal with interrupts */
  int int_pending(void);
  void int_clear(void);
  void int_enable(void);
  void int_disable(void);
  int set_int_level(char level);
  int get_int_level(void);
  void set_int_source(char source);
    
  /* AMD9513A Specific Functions */
  /*****************************************/
  /* Functions that issue counter commands */
  int reset_timers(void);
  int arm_timers(int timer, ...);
  int load_timers(int timer, ...);
  int load_arm_timers(int timer, ...);
  int disarm_timers(int timer, ...);
  int disarm_save_timers(int timers, ...);
  int save_timers(int timers, ...);
  int step_timer(int timer);
  int set_tc(int bit);
  
  /* Functions that deal with load, hold and alarm registers */
  int set_load(int timer, short unsigned int value);
  int set_hold(int timer, short unsigned int value);
  int set_alarm(int timer, short unsigned int value);
  int get_load(int timer, short unsigned int *value);
  int get_hold(int timer, short unsigned int *value);
  int get_alarm(int timer, short unsigned int *value);
  
  /* Functions that deal with the counter mode registers */
  int set_gating(int timer, char gating);
  int set_source(int timer, char source);
  int set_source_edge(int timer, char edge);
  int set_sp_gate(int timer, unsigned char sp_gate);
  int set_reload(int timer, char reload);
  int set_repeat(int timer, char repeat);
  int set_bin_bcd(int timer, char mode);
  int set_direction(int timer, char direction);
  int set_output(int timer, char output_mode);
  int set_master_mode(unsigned int value);
  int get_master_mode(unsigned int *value);
  
  /* Functions that deal with the master mode register */
  int set_fout_divider(char divider);
  int set_fout_source(char source);
  int set_fout_gating(char fout_gate);
  int set_division(unsigned char division);
  int set_comparator1(char comp_mode);
  int set_comparator2(char comp_mode);
  
  /* High level functions */
  int set_mode(int timer, unsigned char mode, unsigned char source,
	       unsigned char source_edge, unsigned char gating,
	       unsigned char output_mode);
  int generate_freq(int timer, long int freq, double dc);	   

  /* Function to convert signed 2's comp numbers */
  int resolve(int test);
};


class DAS20 : public DAS20CL
{
};

#endif /* DAS20_HH */
