#include "das20.hh"
#include <stdio.h>
#include <stdarg.h>

#if (defined(__MSDOS__) || defined(__WIN32__))
#include <stdlib.h>
#include <conio.h>
#endif

#ifdef __MSDOS__
#include <dos.h>
#endif

#ifdef __linux__

#include <unistd.h>	      /* iopl() */

#ifndef USR_PORT_ACCESS

#define extern
#include <asm/io.h>	      /* outw, inw */

/* Map DOS functions to Linux using outb, inb */
#define _outp(port,val) outb(val,port)
#define outp(port,val) outb(val,port)
#define _inp(port) inb(port)
#define inp(port) inb(port)
#define _inpw(port) inw(port)
#define _outpw(port,val) outw(val,port)

#else /* USR_PORT_ACCESS */

#include <sys/types.h> 
#include <sys/stat.h> 
#include <fcntl.h>

#define CHECK_IF_ACCESS_ERROR(port, fd)                       \
        if ( (port < BASE_ADDR) || (port > BASE_ADDR+0x0F) )  \
          {                                                   \
            fprintf(stderr, "ERROR: Attempt to access "       \
                    "restricted port address.\n");            \
	    close(fd);	         			      \
            exit(-1);                                         \
	  }

#define SET_PORT_ADDRESS(port, fd)                            \
        if ( lseek(fd, port, SEEK_SET) == -1 )                \
          {                                                   \
            fprintf(stderr, "ERROR: Cannot access port "      \
                    "number %d.\n", (int) port);              \
            close(fd);                                        \
            exit(-3);                                         \
          }

#define READ_FROM_PORT(fd, val, num_bytes)                    \
        if ( read(fd, val, num_bytes) <= 0 )                  \
          {                                                   \
            fprintf(stderr, "Cannot read from port.\n");      \
            close(fd);                                        \
            exit(-4);                                         \
          }

#define WRITE_TO_PORT(fd, val, num_bytes)                     \
        if ( write(fd, val, num_bytes) <= 0 )                 \
          {                                                   \
            fprintf(stderr, "Cannot write to port.\n");       \
            close(fd);                                        \
            exit(-5);                                         \
          }

/* map DOS funcs to Linux using dev/port */
void _outp( const off_t port, const char val )
{
  CHECK_IF_ACCESS_ERROR(port,fdio_port);
  SET_PORT_ADDRESS(port, fdio_port);
  WRITE_TO_PORT(fdio_port, &val, 1);
}

void outp( const off_t port, const char val )
{
  CHECK_IF_ACCESS_ERROR(port,fdio_port);
  SET_PORT_ADDRESS(port, fdio_port);
  WRITE_TO_PORT(fdio_port, &val, 1);
}

char _inp( const off_t port )
{
  char buf;

  CHECK_IF_ACCESS_ERROR(port, fdio_port);
  SET_PORT_ADDRESS(port, fdio_port);
  READ_FROM_PORT(fdio_port, &buf, 1);
  return(buf);
}

char inp( const off_t port )
{
  char buf;

  CHECK_IF_ACCESS_ERROR(port, fdio_port);
  SET_PORT_ADDRESS(port, fdio_port);
  READ_FROM_PORT(fdio_port, &buf, 1);
  return(buf);
}

unsigned short int _inpw(const off_t port)
{
  unsigned short int buf;

  CHECK_IF_ACCESS_ERROR(port, fdio_port);
  SET_PORT_ADDRESS(port, fdio_port);
  READ_FROM_PORT(fdio_port, &buf, 2);
  return(buf);
}

void _outpw( const off_t port, const int val )
{
  CHECK_IF_ACCESS_ERROR(port, fdio_port);
  SET_PORT_ADDRESS(port, fdio_port);
  WRITE_TO_PORT(fdio_port, &val, 2);
}
  
#endif /* ALTACCESS */
#endif /* __linux__ */

#if (defined(__BORLANDC__) || defined(__BCPLUSPLUS__))
/* Map MSVC to BC functions */
#define _outp(port,val) outportb(port,val)
#define outp(port,val) outportb(port,val)
#define _inp(port) inportb(port)
#define inp(port) inportb(port)
#define _inpw(port) inpw(port)
#define _outpw(port,val) outpw(port,val)

#endif /* __BORLAND */


/*
 * Function Implementations for the DAS-20
 */

DAS20CL::DAS20CL( void )
{
#ifdef __linux__
#ifdef USR_PORT_ACCESS
  if ( (fdio_port = open("/dev/port", O_RDWR)) == -1 )
    {
      fprintf(stderr, "Cannot access /dev/port.\n");
      exit(-1);
    }
#elif !defined(D20RUN)
  if (ioperm(BASE_ADDR, 0x0F, 1) == -1)
    {
      fprintf(stderr, "Cannot set I/O permission.\n");
      exit(-1);
    }
#endif
#endif
}

DAS20CL::~DAS20CL( void )
{
#ifdef __linux__
#ifdef USR_PORT_ACCESS
  close(fdio_port);
#else
  ioperm(BASE_ADDR, 0x0F, 0);
#endif
#endif
}


void DAS20CL::das20_master_reset( void ) 
{
  reset_timers();
  set_digital_byte(0);
  single_dac(0, 0.0);
  single_dac(1, 0.0);
  int_disable();
  _outp(CONTROL, 0x00);
  return;
}

/***************************************************************/

void DAS20CL::set_digital_byte( char value ) 
{
  _outp(DIGITAL_OUT, value);
  return;
}

/***************************************************************/

void DAS20CL::set_digital_bit( short int bit, short int value ) 
{
  char temp;
  
  temp = get_digital_out_byte();
  temp = temp & ~(0x01 << bit);
   temp += value*(0x01 << bit);
  set_digital_byte(temp);
  return;
}

/***************************************************************/

void DAS20CL::toggle_digital_byte( void ) 
{
  set_digital_byte(get_digital_out_byte() ^ 0xFF);
  return;
}

/***************************************************************/

void DAS20CL::toggle_digital_bit( short int bit ) 
{
  set_digital_byte(get_digital_out_byte() ^ (0x01 << bit));
  return;
}

/***************************************************************/

char DAS20CL::get_digital_out_byte( void ) 
{
  return(_inp(DIGITAL_OUT));
}

/***************************************************************/

int DAS20CL::get_digital_out_bit( short int bit ) 
{
  return(get_digital_out_byte() & (0x01 << bit) ? 1 : 0);
}

/***************************************************************/

char DAS20CL::get_digital_in_byte( void ) 
{
  return(_inp(DIGITAL_IN));
}

/***************************************************************/

int DAS20CL::get_digital_in_bit( short int bit ) 
{
  return(get_digital_in_byte() & (0x01 << bit));
}

/***************************************************************/

double DAS20CL::single_adc( short int channel, short int range ) 
{
  char c[1], r[1];
  int input;
  int old_int_status;
  
  c[0] = channel;
  r[0] = range;
  
  old_int_status = _inp(INT_STATUS);
  /* Interrupt flag generated on ADC EOC */
  _outp(INT_CONTROL, (INT_DISABLE + INT_ADC_EOC));
  /* ADC started by software */
  _outp(CONTROL, (_inp(CONTROL) & 0xF3));
  /* Load ADC queue for one sample */
  load_queue(c, r, 1);
  /* Start conversion */
  _outp(ADC_START, 0x00);    
  
  /* Wait for EOC */
  while ((_inp(INT_STATUS) & INT_MASK) != INT_MASK); 
  
  /* Clear interrupt flag */
  _outp(INT_CONTROL, 0x00);

  /* Restore old interrupt status. */
  _outp(INT_CONTROL, old_int_status);

  /* Get results without channel info */
  input = _inpw(ADC_LO) >> 4;
  
  /* Convert to a real number based on the range selected */
  switch(range) 
    {
    case 1 : return(((double)input / 4096.0) * 10.0);
    case 2 : return(((double)resolve(input) / 2048.0) * 10.0);
    case 3 : return(((double)input / 4096.0) * 10.0);
    case 4 : return(((double)resolve(input) / 2048.0) * 5.0);
    case 5 : return((double)input / 4096.0);
    case 6 : return(((double)resolve(input) / 2048.0) * 0.5);
    case 7 : return(((double)input / 4096.0) * 0.1);
    case 8 : return(((double)resolve(input) / 2048.0) * 0.05);
    default: return(((double)input / 4096.0) * 10.0);
    }
}               

/***************************************************************/

int DAS20CL::resolve( int test ) 
{
  if ((test & 0x800) == 0x800) 
    {
      test = ~test;
      test++;
      test = (test & 0xFFF) * -1;
    }
  else
    test = test & 0xFFF;
  
  return(test);
}

/***************************************************************/

void DAS20CL::load_queue( char chan[], char range[], int number ) 
{
  char temp, gain, channel;
  int i;
  
  /* Set queue to be loaded */
  temp = _inp(CONTROL);
  temp = (temp & 0x3F) + ADC_Q_LOAD;
  _outp(CONTROL, temp);
  
  for (i = 0; i < number; i++) 
    {
      /* Find the range bits corresponding to range number */
      switch(range[i]) 
	{
	case 1 : gain = RANGE1; break;
	case 2 : gain = RANGE2; break;
	case 3 : gain = RANGE3; break;
	case 4 : gain = RANGE4; break;
	case 5 : gain = RANGE5; break;
	case 6 : gain = RANGE6; break;
	case 7 : gain = RANGE7; break;
	case 8 : gain = RANGE8; break;
	default: gain = RANGE1; break;
	}
      channel = chan[i] << 4;  /* ADC channel info is in hi nybble */
      _outp(SCANQ_LOAD, (channel + gain) & 0xFF);  /* Load into queue */
    }
  
  /* Output end of queue flag */
  _outp(SCANQ_LOAD, QUEUE_END);
  
  /* Set queue to be read */
  temp = _inp(CONTROL);
  temp = (temp & 0x3F) + ADC_Q_ENABLE + ADC_Q_WRAP;
  _outp(CONTROL, temp);
  /* Reset queue pointer to begining of queue */
  temp = _inp(CONTROL);
  
  return;
}

/***************************************************************/


int DAS20CL::single_dac( short int channel, double value ) 
{
  char dac_range;
  int dac_lo, dac_hi, range, output;
  
  /* Exit if channel number is incorrect */
  if ((channel != 0) && (channel != 1))
    return(-1);
  
  /* Get gain and range info */
  dac_range = _inp(DMA_CONTROL);
  if (channel == 0) 
    {
      dac_range = dac_range & 0x05; /* Mask off unused bits for comparason */
      dac_lo = DAC0_LO;             /* Port for DAC0 */
      dac_hi = DAC0_HI;
    }
  else 
    {
      dac_range = (dac_range & 0x0A) >> 1;   /* Make DAC1 look like DAC0 */
      dac_lo = DAC1_LO;
      dac_hi = DAC1_HI;
    }
  
  /* Find the gain and range of channel */
  if (dac_range == 0x00)
    range = 1;     /* 0 to +10 volts */
  else if (dac_range == 0x01)
    range = 5;     /* -5 to +5 volts */
  else if (dac_range == 0x04) 
    {
      range = -1;
      return(-1);
    }
  else
    range = 10;    /* -10 to +10 volts */
  
  /* Determine output based on value and range */
  /* For 0 to 10 range */
  if (range == 1) 
    {
      /* Output is 0 volts if value < 0.0 */
      if (value < 0.0) 
	{
	  _outp(dac_lo, 0x00);
	  _outp(dac_hi, 0x00);
	  return(-5);
	}
      /* Output is 10 volts if value > 10.0 */
      else if (value > 10.0) 
	{
	  _outp(dac_lo, 0xFF);
	  _outp(dac_hi, 0xFF);
	  return(5);
	}
      /* Output is value if 0.0 <= value <= 10.0 */
      else 
	{
	  output = (int)(value*4096.0/10.0);
	  _outp(dac_lo, (output & 0xFF));
	  _outp(dac_hi, (output & 0xF00) >> 8);
	  return(0);
	}
    }
  /* For -5 to 5 and -10 to 10 ranges */
  else 
    {
      /* Output is -range if value < -range */
      if (value < -(double)range) 
	{
	  _outp(dac_lo, 0x00);
	  _outp(dac_hi, 0x08);
	  return(-5);
	}
      /* Output is range if value > range */
      else if (value > (double)range) 
	{
	  _outp(dac_lo, 0xFF);
	  _outp(dac_hi, 0x07);
	  return(5);
	}
      /* Output is value if -range <= value <= range */
      else 
	{
	  output = (int)(value*2047.0/(double)range);
	  _outp(dac_lo, (output & 0xFF));
	  _outp(dac_hi, (output & 0xF00) >> 8);
	  return(0);
	}
    }
}

/***************************************************************/

int DAS20CL::int_pending ( void ) 
{
  return(_inp(INT_STATUS) & 0x02);
}

/***************************************************************/

void DAS20CL::int_clear (void) 
{
  _outp(INT_CONTROL, (_inp(INT_STATUS) & 0xFD));
  return;
}

/***************************************************************/

void DAS20CL::int_enable( void ) 
{
  _outp(INT_CONTROL, (_inp(INT_STATUS) | 0x80));
  return;
}

/***************************************************************/

void DAS20CL::int_disable( void ) 
{
  _outp(INT_CONTROL, (_inp(INT_STATUS) & 0x7F));
  return;
}

/***************************************************************/

int DAS20CL::set_int_level(char level) 
{
  char temp;
  
  if ((level < 2) || (level > 7))
    return(-1);
  
  temp = _inp(INT_STATUS);
  temp += (level << 4) & 0x70;
  _outp(INT_CONTROL, temp);
  
  return(0);
}

/***************************************************************/

int DAS20CL::get_int_level( void ) 
{
  return((_inp(INT_STATUS) >> 4) & 0x07);
}

/***************************************************************/

void DAS20CL::set_int_source( char source ) 
{
  _outp(INT_CONTROL, (_inp(INT_CONTROL) & 0xF3) + source);
  return;
}

/***************************************************************/



/***************************************************************
 * Function Implementations for the AMD9513A system timer
 ***************************************************************/

int DAS20CL::reset_timers(void) 
{
  _outp(TMR_CONTROL, MASTER_RESET);
  return(0);
}

/***************************************************************/

int DAS20CL::arm_timers (int timer, ...) 
{
  va_list timer_ptr;
  unsigned char output = 0;
  
  if (timer != 0) 
    {
      output += (0x01 << (timer - 1));
      va_start(timer_ptr, timer);
      while((timer = va_arg(timer_ptr, int)) != 0) 
	{
	  if ((timer < 1) || (timer > 5))
	    return(-1);
	  else
	    output += (0x01 << (timer - 1));
	}
      _outp(TMR_CONTROL, (ARM_TMR + output));
      va_end(timer_ptr);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::load_timers (int timer, ...) 
{
  va_list timer_ptr;
  unsigned char output = 0;
  
  if (timer != 0) 
    {
      output += (0x01 << (timer - 1));
      va_start(timer_ptr, timer);
      while((timer = va_arg(timer_ptr, int)) != 0) 
	{
	  if ((timer < 1) || (timer > 5))
	    return(-1);
	  else
	    output += (0x01 << (timer - 1));
	}
      _outp(TMR_CONTROL, (LOAD_TMR + output));
      va_end(timer_ptr);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::load_arm_timers (int timer, ...) 
{
  va_list timer_ptr;
  unsigned char output = 0;
  
  if (timer != 0) 
    {
      output += (0x01 << (timer - 1));
      va_start(timer_ptr, timer);
      while((timer = va_arg(timer_ptr, int)) != 0) 
	{
	  if ((timer < 1) || (timer > 5))
	    return(-1);
	  else
	    output += (0x01 << (timer - 1));
	}
      _outp(TMR_CONTROL, (LOAD_ARM_TMR + output));
      va_end(timer_ptr);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::disarm_timers (int timer, ...) 
{
  va_list timer_ptr;
  unsigned char output = 0;
  
  if (timer != 0) 
    {
      output += (0x01 << (timer - 1));
      va_start(timer_ptr, timer);
      while((timer = va_arg(timer_ptr, int)) != 0) 
	{
	  if ((timer < 1) || (timer > 5))
	    return(-1);
	  else
	    output += (0x01 << (timer - 1));
	}
      _outp(TMR_CONTROL, (DISARM_TMR + output));
      va_end(timer_ptr);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::save_timers (int timer, ...) 
{
  va_list timer_ptr;
  unsigned char output = 0;
  
  if (timer != 0) 
    {
      output += (0x01 << (timer - 1));
      va_start(timer_ptr, timer);
      while((timer = va_arg(timer_ptr, int)) != 0) 
	{
	  if ((timer < 1) || (timer > 5))
	    return(-1);
	  else
	    output += (0x01 << (timer - 1));
	}
      _outp(TMR_CONTROL, (SAVE_TMR + output));
      va_end(timer_ptr);
    }
  return(0);
}

/***************************************************************/

int disarm_save_timers (int timer, ...) 
{
  va_list timer_ptr;
  unsigned char output = 0;
  
  if (timer != 0) 
    {
      output += (0x01 << (timer - 1));
      va_start(timer_ptr, timer);
      while((timer = va_arg(timer_ptr, int)) != 0) 
	{
	  if ((timer < 1) || (timer > 5))
	    return(-1);
	  else
	    output += (0x01 << (timer -1));
	}
      _outp(TMR_CONTROL, (DISARM_SAVE_TMR + output));
      va_end(timer_ptr);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::step_timer(int timer) 
{
  if ((timer < 1) || (timer > 5))
    return(-1);
  else
    _outp(TMR_CONTROL, (STEP_TMR + timer));
  
  return(0);
}

/***************************************************************/

int DAS20CL::set_tc(int bit) 
{
  if ((bit < 0) || (bit > 1))
    return(-1);
  else 
    {
      if (bit == 0)
	_outp(TMR_CONTROL, CLEAR_TC_TOGGLE);
      else
	_outp(TMR_CONTROL, SET_TC_TOGGLE);
    }
  
  return(0);
}

/***************************************************************/

int DAS20CL::set_load(int timer, short unsigned int value) 
{
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + LOAD_REG));
      _outp(TMR_DATA, (value & 0xFF));
      _outp(TMR_DATA, ((value & 0xFF00) >> 8));
    }
  
  return(0);
}

/***************************************************************/

int DAS20CL::set_hold(int timer, short unsigned int value) 
{
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + HOLD_REG));
      _outp(TMR_DATA, (value & 0xFF));
      _outp(TMR_DATA, ((value & 0xFF00) >> 8));
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_alarm(int timer, short unsigned int value) 
{
  
  if ((timer < 1) || (timer > 2))
    return(-1);
  else if (timer == 1) 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + ALARM_REG1));
      _outp(TMR_DATA, (value & 0xFF));
      _outp(TMR_DATA, ((value & 0xFF00) >> 8));
    }
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + ALARM_REG2));
      _outp(TMR_DATA, (value & 0xFF));
      _outp(TMR_DATA, ((value & 0xFF00) >> 8));
    }
  return(0);
}

/***************************************************************/

int DAS20CL::get_load(int timer, short unsigned int *value) 
{
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + LOAD_REG));
      *value = _inp(TMR_DATA);
      *value += (_inp(TMR_DATA) << 8);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::get_hold(int timer, short unsigned int *value) 
{
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + HOLD_REG));
      *value = _inp(TMR_DATA);
      *value += (_inp(TMR_DATA) << 8);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::get_alarm(int timer, short unsigned int *value) 
{
  if ((timer < 1) || (timer > 2))
    return(-1);
  else if (timer == 1) 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + ALARM_REG1));
      *value = _inp(TMR_DATA);
      *value += (_inp(TMR_DATA) << 8);
    }
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + ALARM_REG2));
      *value = _inp(TMR_DATA);
      *value += (_inp(TMR_DATA) << 8);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_gating(int timer, char gating) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = _inp(TMR_DATA);
      temp_hi = (_inp(TMR_DATA) & 0x1F) + gating;
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_source(int timer, char source) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = _inp(TMR_DATA);
      temp_hi = (_inp(TMR_DATA) & 0xF0) + source;
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_source_edge(int timer, char edge) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = _inp(TMR_DATA);
      temp_hi = (_inp(TMR_DATA) & 0xEF) + edge;
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_sp_gate(int timer, unsigned char sp_gate) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = (_inp(TMR_DATA) & 0x7F) + sp_gate;
      temp_hi = _inp(TMR_DATA);
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_reload(int timer, char reload) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = (_inp(TMR_DATA) & 0xBF) + reload;
      temp_hi = _inp(TMR_DATA);
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_repeat(int timer, char repeat) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = (_inp(TMR_DATA) & 0xDF) + repeat;
      temp_hi = _inp(TMR_DATA);
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_bin_bcd(int timer, char mode) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = (_inp(TMR_DATA) & 0xEF) + mode;
      temp_hi = _inp(TMR_DATA);
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_direction(int timer, char direction) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = (_inp(TMR_DATA) & 0xF7) + direction;
      temp_hi = _inp(TMR_DATA);
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_output(int timer, char output_mode) 
{
  unsigned char temp_lo, temp_hi;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  else 
    {
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      temp_lo = (_inp(TMR_DATA) & 0xF8) + output_mode;
      temp_hi = _inp(TMR_DATA);
      _outp(TMR_CONTROL, (LOAD_DPR + timer + MODE_REG));
      _outp(TMR_DATA, temp_lo);
      _outp(TMR_DATA, temp_hi);
    }
  return(0);
}

/***************************************************************/

int DAS20CL::set_fout_divider(char divider) 
{
  unsigned char temp_lo, temp_hi;
  
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  temp_lo = _inp(TMR_DATA);
  temp_hi = (_inp(TMR_DATA) & 0xF0) + divider;
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, temp_lo);
  _outp(TMR_DATA, temp_hi);
  return(0);
}

/***************************************************************/

int DAS20CL::set_fout_source(char source) 
{
  unsigned char temp_lo, temp_hi;
  
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  temp_lo = (_inp(TMR_DATA) & 0x0F) + source;
  temp_hi = _inp(TMR_DATA);
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, temp_lo);
  _outp(TMR_DATA, temp_hi);
  return(0);
}

/***************************************************************/

int DAS20CL::set_fout_gating(char fout_gate) 
{
  unsigned char temp_lo, temp_hi;
  
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  temp_lo = _inp(TMR_DATA);
  temp_hi = (_inp(TMR_DATA) & 0xEF) + fout_gate;
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, temp_lo);
  _outp(TMR_DATA, temp_hi);
  return(0);
}

/***************************************************************/

int DAS20CL::set_division(unsigned char division) 
{
  unsigned char temp_lo, temp_hi;
  
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  temp_lo = _inp(TMR_DATA);
  temp_hi = (_inp(TMR_DATA) & 0x7F) + division;
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, temp_lo);
  _outp(TMR_DATA, temp_hi);
  return(0);
}

/***************************************************************/

int DAS20CL::set_comparator1(char comp_mode) 
{
  unsigned char temp_lo, temp_hi;
  
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  temp_lo = (_inp(TMR_DATA) & 0xFB) + comp_mode;
  temp_hi = _inp(TMR_DATA);
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, temp_lo);
  _outp(TMR_DATA, temp_hi);
  return(0);
}

/***************************************************************/

int DAS20CL::set_comparator2(char comp_mode) 
{
  unsigned char temp_lo, temp_hi;
  
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  temp_lo = (_inp(TMR_DATA) & 0xF7) + comp_mode;
  temp_hi = _inp(TMR_DATA);
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, temp_lo);
  _outp(TMR_DATA, temp_hi);
  return(0);
}

/***************************************************************/

int DAS20CL::set_master_mode(unsigned int value) 
{
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  _outp(TMR_DATA, (value & 0xFF));
  _outp(TMR_DATA, ((value & 0xFF00) >> 8));
  return(0);
}

/***************************************************************/

int DAS20CL::get_master_mode(unsigned int *value) 
{
  _outp(TMR_CONTROL, (LOAD_DPR + CONTROL_GROUP + MASTER_MODE_REG));
  *value = _inp(TMR_DATA);
  *value += _inp(TMR_DATA) << 8;
  return(0);
}

/***************************************************************/

int DAS20CL::set_mode(int timer, unsigned char mode, unsigned char source,
		      unsigned char source_edge, unsigned char gating,
		      unsigned char output_mode) 
{
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  
  mode = mode | 0x20;   /* Everything in lower case */
  
  /* Set these options if desired */
  if (source != 0xFF) set_source(timer, source);
  if (source_edge != 0xFF) set_source_edge(timer, source_edge);
  if (gating != 0xFF) set_gating(timer, gating);
  if (output_mode != 0xFF) set_output(timer, output_mode);
  
  switch (mode) 
    {
    case 'a':
      set_gating(timer, NO_GATE);
    case 'b':
    case 'c':
      set_sp_gate(timer, DISABLE_SP_GATE);
      set_reload(timer, RELOAD_LOAD);
      set_repeat(timer, COUNT_ONCE);
      break;
    case 'd':
      set_gating(timer, NO_GATE);
    case 'e':
    case 'f':
      set_sp_gate(timer, DISABLE_SP_GATE);
      set_reload(timer, RELOAD_LOAD);
      set_repeat(timer, COUNT_REPEAT);
      break;
    case 'g':
      set_gating(timer, NO_GATE);
    case 'h':
    case 'i':
      set_sp_gate(timer, DISABLE_SP_GATE);
      set_reload(timer, RELOAD_EITHER);
      set_repeat(timer, COUNT_ONCE);
      break;
    case 'j':
      set_gating(timer, NO_GATE);
    case 'k':
    case 'l':
      set_sp_gate(timer, DISABLE_SP_GATE);
      set_reload(timer, RELOAD_EITHER);
      set_repeat(timer, COUNT_REPEAT);
      break;
    case 'n':
    case 'o':
      set_sp_gate(timer, ENABLE_SP_GATE);
      set_reload(timer, RELOAD_LOAD);
      set_repeat(timer, COUNT_ONCE);
      break;
    case 'q':
    case 'r':
      set_sp_gate(timer, ENABLE_SP_GATE);
      set_reload(timer, RELOAD_LOAD);
      set_repeat(timer, COUNT_REPEAT);
      break;
    case 's':
      set_sp_gate(timer, ENABLE_SP_GATE);
      set_reload(timer, RELOAD_EITHER);
      set_repeat(timer, COUNT_ONCE);
      break;
    case 'v':
    case 'x':
      set_sp_gate(timer, ENABLE_SP_GATE);
      set_reload(timer, RELOAD_EITHER);
      set_repeat(timer, COUNT_REPEAT);;
      break;
    default: /* Default is mode a */
      set_gating(timer, NO_GATE);
      set_sp_gate(timer, DISABLE_SP_GATE);
      set_reload(timer, RELOAD_LOAD);
      set_repeat(timer, COUNT_ONCE);
      break;
    }  /* End of switch */
  
  return(0);
}

/***************************************************************/

int DAS20CL::generate_freq(int timer, long int freq, double dc) 
{
  int count, load_count, hold_count;
  
  if ((timer < 1) || (timer > 5))
    return(-1);
  
  if ((freq < 1) || (freq > 500000L))
    return(-2);
  
  if ((dc <= 0) || (dc >= 1))
    return(-3);
  set_division(BCD_DIVISION);
  
  if (freq >= 100) 
    {
      count = (int)(5000000L / freq);
      hold_count = (int)(count * dc);
      load_count = count - hold_count;
      
      disarm_timers(timer, 0);
      set_load(timer, load_count);
      set_hold(timer, hold_count);
      set_mode(timer, 'j', F1, RISE_SOURCE, NO_GATE, TC_TOGGLE);
      load_timers(timer, 0);
    }
  else 
    {
      count = (int)(50000L / freq);
      hold_count = (int)(count * dc);
      load_count = count - hold_count;
      
      disarm_timers(timer, 0);
      set_load(timer, load_count);
      set_hold(timer, hold_count);
      set_mode(timer, 'j', F3, RISE_SOURCE, NO_GATE, TC_TOGGLE);
      load_timers(timer, 0);
    }
  return(0);
}

/***************************************************************
 * End of file das20.cc
 ***************************************************************/
