#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <asm/io.h>

#include "das20.h" 

#define SAMPLE_TIME 0.001 
#define RUN_TIME 15.0

int fpurge(FILE *);

int main(void) 
{ 
  double time, output;
  double value[3];
  unsigned char digValue;
  int aa;
  long int cycles=0;
  clock_t start, stop;

  fprintf(stdout, "Starting program....\n");
  fprintf(stdout, "Reseting das20 board....\n");
  /* Reset Data Acquisition Board */
  das20_master_reset();

  /* Input value to set output of D/A Ch 1 In this program output of
     D/A Ch 1 is connected to A/D Ch 0 and 1 to test A/D conversion */
  fprintf(stdout, "Enter a value to send to D/A Channel 1 (in Volts): ");
  scanf("%lf",&output);
  
  /* Generate sampling timer */
  generate_freq(2, 1/SAMPLE_TIME, 0.50);
  /* Set interrupt source to be the sampling timer */
  set_int_source(INT_TMR2);

  time = 0.0; 

  /* Start timing */
  arm_timers(2, 0); 

  /* Test sampling timer with system time */
  start = clock();

  /* MAIN LOOP */
  while (time <= RUN_TIME)
    { 
      /* Determine output voltage and place it on DAC1 */
      single_dac(1, output);
      
      /* Do nothing until sampling time */
      while (int_pending() == 0);    
      /* Clear interrupt flag */
      int_clear(); 
      
      time = time + SAMPLE_TIME;
      cycles++; 
      if (cycles % (int)(1/SAMPLE_TIME) == 0)
	{  
	  /* Read in values of A/D Ch 0 and 1 (should be = output) */
	  value[1] = single_adc(0,1);
	  value[2] = single_adc(1,1);
	  fprintf(stdout, "Time = %3.0f sec.  Value 1 and 2 = %6.3f %6.3f"
		  " (these should be equal to %6.3f)\n",
		  time, value[1], value[2], output);
	  fflush(stdout);
	}
    } /* Ends while (... */

  stop = clock();
  fprintf(stdout, "Average time per cycle: %f => %f Hz\n", 
	  (double) (stop-start) / (cycles*CLOCKS_PER_SEC),
	  (double) (cycles*CLOCKS_PER_SEC) / (stop-start));
  
  /* Test digital functions */
  set_digital_byte(0x0F);
  digValue = get_digital_in_byte();
  fprintf(stdout, "\nDigOut should read 0000 1111\n");
  fprintf(stdout, "In hexadecimal this is %X. Hit key to continue.\n", 
	 digValue);
  fpurge(stdin);
  getchar();
  
  for (aa=0; aa<8; aa++)
    if (aa < 4)
      set_digital_bit(aa,0);
    else
      set_digital_bit(aa,1);
  
  digValue = get_digital_in_byte();
  fprintf(stdout, "\nDigOut should read 1111 0000\n");
  fprintf(stdout, "In hexadecimal this should be %X. Hit key to continue.\n", 
	 digValue);
  fpurge(stdin);
  getchar(); 
  
  toggle_digital_byte();
  digValue = get_digital_in_byte();
  fprintf(stdout, "\nDigOut should read 0000 1111\n");
  fprintf(stdout, "In hexadecimal this should be %X. Hit key to continue.\n", 
	 digValue);
  fpurge(stdin);
  getchar();
  
  toggle_digital_bit(0);
  toggle_digital_bit(4);
  digValue = get_digital_in_byte();
  fprintf(stdout, "\nDigOut should read 0001 1110\n");
  fprintf(stdout, "In hexadecimal this should be %X. Hit key to end.\n", 
	  digValue);
  fpurge(stdin);
  getchar();
  
  /* Clean up and quit */
  das20_master_reset(); 
  das20_cleanup();

  return 0;
} 	      


int fpurge(FILE *stream)
{
  fflush(stream);
  return( fseek(stream, 0L, SEEK_END) );
}
